#!/usr/bin/env php
<?php

/**
 * Observium
 *
 *   This file is part of Observium.
 *
 * @package    observium
 * @subpackage scripts
 * @copyright  (C) 2006-2013 Adam Armstrong, (C) 2013-2018 Observium Limited
 *
 */

chdir(dirname($argv[0]).'/..');
$scriptname = basename($argv[0]);

$options = getopt("d");
if (isset($options['d'])) { array_shift($argv); } // for compatability

include_once("includes/sql-config.inc.php");

//$config['rancid_version'] = '3.7.1';

if (isset($config['rancid_version']) && str_starts($config['rancid_version'], '3'))
{
  // Rancid version configured and more than 2
  $rancid_version = ltrim($config['rancid_version'], 'vV');
  $rancid_message = "Used RANCID version configured manually in \$config['rancid_version']: $rancid_version";
  // v3 delimiter
  $rancid_version_base = '3';
  $delimiter = ';';
}
else if ($rancid_cmd = external_exec('which rancid-run'))
{
  // Try detect rancid version on local system
  list(, $rancid_version) = explode(' ', external_exec($rancid_cmd . ' -V'));
  $rancid_message = "Used RANCID version detected on system: $rancid_version";
  list($rancid_version_base) = explode('.', $rancid_version);
  if ($rancid_version_base < 3)
  {
    $delimiter = ':';
  } else {
    $delimiter = ';';
  }
} else {
  // Old version
  $rancid_version = ltrim($config['rancid_version'], 'vV');
  $rancid_message = "Used default RANCID version: $rancid_version. For version 3.x, install rancid locally or use \$config['rancid_version'].";
  // v2 delimiter
  $rancid_version_base = '2';
  $delimiter = ':';
}
print_debug($rancid_message);

?>
# RANCID router.db autogenerated by <?php echo realpath($_SERVER['SCRIPT_FILENAME']) . PHP_EOL; ?>
# <?php echo $rancid_message . PHP_EOL; ?>
# Do not edit this file directly!

<?php

foreach (dbFetchRows("SELECT `hostname`, `os`, `disabled`, `status` FROM `devices` ORDER BY `hostname`") as $device)
{
  if ($device['disabled'] || !$device['status'])
  {
    $status = "down";
  } else {
    $status = "up";
  }

  if (isset($config['rancid']['os_map'][$device['os']]))
  {
    // Common os map for both v2/v3
    echo($device['hostname'] . $delimiter . $config['rancid']['os_map'][$device['os']] . $delimiter . $status . PHP_EOL);
  }

  if ($rancid_version_base > 2)
  {
    $rancid_version = rtrim($rancid_version, '.0');
    // Rancid v3.x
    foreach (array_keys($config['rancid']) as $key)
    {
      if (!str_starts($key, 'os_map_')) { continue; }

      $version = str_replace('os_map_', '', $key); // os_map_3.3 -> 3.3
      $version = rtrim($version, '.0');
      // Compare used rancid version with os map
      if (version_compare($version, $rancid_version, '<=') && isset($config['rancid'][$key][$device['os']]))
      {
        // Version specific os maps
        //echo($device['hostname'] . $delimiter . $config['rancid'][$key][$device['os']] . $delimiter . $status . " $version <= $rancid_version" . PHP_EOL);
        echo($device['hostname'] . $delimiter . $config['rancid'][$key][$device['os']] . $delimiter . $status . PHP_EOL);
      }
    }

  }
}

// EOF
